<?php

namespace Drupal\datahub_webform_handler\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure example settings for this site.
 */
class WebformDataHubConfigForm extends ConfigFormBase {
  /**
  * Config settings.
  *
  * @var string
  */
  const SETTINGS = 'webform_datahub.settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'webform_datahub_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultConfiguration() {
    $config = \Drupal::config('webform_datahub.settings');

    return [
      'endpoint_api' => $config->get('endpoint_api') ? $config->get('endpoint_api') : '',
      'username' => $config->get('username') ? $config->get('username') : '',
      'password_field' => $config->get('password_field') ? $config->get('password_field') : '',
      'accessToken' => $config->get('accessToken') ? $config->get('accessToken') : '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'webform_datahub.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('webform_datahub.settings');

    $form['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#default_value' => $config->get('username'),
      '#required' => TRUE,
    ];

    $form['password_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#default_value' => $config->get('password_field'),
      '#required' => TRUE,
    ];

    $form['accessToken'] = [
      '#type' => 'hidden',
      '#value' => $config->get('accessToken'),
    ];

    $form['endpoint_api'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Endpoint API Key Base URI'),
      '#default_value' => $config->get('endpoint_api'),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $endpoint_api_url = $form_state->getValue('endpoint_api') . "/api/secure/token";
    if (preg_match('/^(https?:\/\/)?([\da-z\.-]+)\.([a-z\.]{2,6})([\/\w \.-]*)*\/?$/', $endpoint_api_url)) {
      if (strpos($endpoint_api_url, "http://") === 0 || strpos($endpoint_api_url, "https://") === 0) {
      }
      else {
        $form_state->setErrorByName('endpoint_api', $this->t('EndPoint API Key protocol is missing'));
      }
    }
    else {
      $form_state->setErrorByName('endpoint_api', $this->t('EndPoint API Key is incorrect'));
    }

    // URL to which you want to make the POST request.
    try {
      $username = $form_state->getValue('username');
      $password_field = $form_state->getValue('password_field');
      $client = \Drupal::httpClient()->post($endpoint_api_url, [
        'headers' => [
      // Adjust the content type as needed.
          'Content-Type' => 'application/json',
          'username' => $username,
          'password' => $password_field,
        ],
      ]);

      $response_body = $client->getBody()->getContents();
      $data = json_decode($response_body, TRUE);

      $accessToken = $data['accessToken'];
      $config = \Drupal::service('config.factory')->getEditable('webform_datahub.settings');
      $config->set('accessToken', $accessToken)
        ->save();
      \Drupal::logger('accessToken form response_body')->notice('<pre><code>' . print_r($response_body, TRUE) . '</code></pre>');
    }
    catch (\Throwable $e) {
      \Drupal::messenger()->addError('Username or password is incorrect');
      $form_state->setErrorByName('username');
      $form_state->setErrorByName('password_field');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $formState) {
    $config = \Drupal::service('config.factory')->getEditable('webform_datahub.settings');
    $config->set('endpoint_api', (string) $formState->getValue('endpoint_api'))
      ->set('username', (string) $formState->getValue('username'))
      ->set('password_field', (string) $formState->getValue('password_field'))
      ->save();
    parent::submitForm($form, $formState);
  }

}
