<?php

namespace Drupal\datahub_webform_handler\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\webform\Entity\WebformSubmission;

/**
 * Webform example handler.
 *
 * @WebformHandler(
 *   id = "webform_datahub",
 *   label = @Translation("WebformDatahub"),
 *   category = @Translation("WebformDatahub"),
 *   description = @Translation("Send the values to the api on
 *   form submission."), cardinality =
 *   \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results =
 *   \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission =
 *   \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 *   tokens = TRUE,
 * )
 */
class WebformDatahub extends WebformHandlerBase {

  /**
   * The user Account.
   *
   * @var \Drupal\user\UserInterface
   */

  /**
   * The entity Field Manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The language Manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $plugin = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $plugin->entityFieldManager = $container->get('entity_field.manager');
    $plugin->languageManager = $container->get('language_manager');
    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'user_field_mapping' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Session\AccountProxyInterface $user */

    // Get webform settings.
    /** @var \Drupal\webform\WebformInterface $webform */
    $webform = $this->getWebform();
    $webform_settings = $webform->getSettings();

    // Retrieve all mapping options:
    // Source  -> destination
    // webform -> user field.
    $mapping_options = $this->getMappingOptions();

    $form['mapping']['user_field_mapping'] = [
      '#type' => 'webform_mapping',
      '#description' => $this->t('Map webform element values to user fields.'),
      '#required' => FALSE,
      '#source' => $mapping_options['source'],
      '#destination' => $mapping_options['destination'],
      '#default_value' => $this->configuration['user_field_mapping'],
      '#source__title' => $this->t('Webform element'),
      '#destination__type' => 'select',
      '#destination__title' => $this->t('User field destination'),
      '#destination__description' => NULL,
      '#parents' => ['settings', 'user_field_mapping'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    parent::validateForm($form, $form_state, $webform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    // Get the mapping between webform elements and user entity properties or
    // fields.
    $user_field_mapping = $form_state->getValue('user_field_mapping', []);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    $submissionID = $webform_submission->id();
    $webform_data = $webform_submission->getData();
    $webform_name = $webform_submission->getWebform()->label();
    $webform_uuid = $webform_submission->getWebform()->uuid();
    $submission = WebformSubmission::load($submissionID);
    $submission_uuid = $submission->get('uuid')->value;
    $user_field_mapping = $this->configuration['user_field_mapping'];

    $user_field_data = [];
    foreach ($user_field_mapping as $webform_key => $user_field) {
      // Grab the value from the webform element and assign it to the correct
      // user field key.
      $user_field_data[$user_field] = $webform_data[$webform_key];
    }
    $response = [];
    $queryParams = [];
    $currentURL = "http" . (isset($_SERVER['HTTPS']) ? "s" : "") . "://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
    // Check if the current URL has query parameters.
    if (!empty($_SERVER['QUERY_STRING'])) {
      $url = $currentURL;

      // Parse the query parameters from the URL.
      $queryString = parse_url($url, PHP_URL_QUERY);
      parse_str($queryString, $queryParams);
    }
    $getAccessToken = \Drupal::service('datahub_webform_handler.GetAccessToken')->accessToken();
    if ($getAccessToken) {
      $data = json_decode($getAccessToken, TRUE);
      $accessToken = $data['accessToken'];
      $submittedValue = \Drupal::service('datahub_webform_handler.BodyValue')->formValues($user_field_data, $webform_uuid, $submission_uuid, $webform_name, $queryParams, $response);
      $apiResponse = \Drupal::service('datahub_webform_handler.DatahubIntegration')->sentWebformSubmissionTodatahub($submittedValue, $accessToken);
    }
    return $user_field_data;
    // $this->remotePost($state, $webform_submission);
  }

  /**
   * Helper function to return a array of source and destination options.
   *
   * Source options would contain possible webform elements.
   * Destination options would contain user entity properties and fields:
   * [
   *   'source' => [
   *     'first_name'
   *     'last_name'
   *     ...
   *   ],
   *   'destination' => [
   *     'name'
   *     'mail'
   *     ...
   *   ]
   * ].
   *
   * @return array
   *   Array of source and destination options.
   */
  protected function getMappingOptions() {
    $source_options = [];
    $destination_options = [];

    // Load all webform elements.
    /** @var \Drupal\webform\Plugin\WebformElementInterface[] $webform_elements */
    $webform_elements = $this->webform->getElementsInitializedFlattenedAndHasValue();
    foreach ($webform_elements as $key => $element) {
      $source_options[$key] = $element['#admin_title'] ?: $element['#title'] ?: $key;
    }

    // Load all user entity fields.
    /** @var \Drupal\Core\Entity\EntityFieldManagerInterface $entity_storage */
    $user_fields = ['firstName',
      'email',
      'surname',
      'websiteURL',
      'phoneNumberBaseNumber',
      'addressCountry',
      'registrationStatus',
      'jobTitle',
      'addressTownCity',
      'title',
      'gender',
      'companyName',
      'interest',
      'industry',
      'eventEditionCode',
      'getInTouch',
      'eventsAndProduct',
      'CPHIMilanEvents',
      'CPHIMilanLatestProductsServices',
      'CPHIMilanPartnersProductsServices',
      'privacyTerms',
      'visitorType',
      'jobFunction',
    ];
    /** @var string $key */
    /** @var \Drupal\Core\Field\FieldDefinitionInterface $field */
    foreach ($user_fields as $key => $field) {
      $destination_options[$field] = $field;
    }
    return [
      'source' => $source_options,
      'destination' => $destination_options,
    ];
  }

}
